#include "pico/stdlib.h"

#define CONTACTEUR_ACTIF 0
#define CONTACTEUR_INACTIF 1

void i2c_annexe_gestion(void);
void i2c_annexe_active_turbine(void);
void i2c_annexe_desactive_turbine(void);

void i2c_annexe_ouvre_porte(void);
void i2c_annexe_ferme_porte(void);

void i2c_annexe_active_propulseur(void);
void i2c_annexe_desactive_propulseur(void);

uint8_t i2c_annexe_get_contacteur_butee_A(void);
uint8_t i2c_annexe_get_contacteur_butee_C(void);

uint8_t i2c_annexe_get_contacteur_longer_A(void);
uint8_t i2c_annexe_get_contacteur_longer_C(void);

void i2c_annexe_get_distances(uint8_t *distance_capteur_cm);